@@ -18,8 +18,6 @@ import asyncio
 import inspect
 import logging
 import ssl
-import sys
-import random
 from datetime import datetime, timedelta, timezone
 from functools import partial
 from http import HTTPStatus
@@ -43,7 +41,7 @@ class OpenADRClient:
     """
     def __init__(self, ven_name, vtn_url, debug=False, cert=None, key=None,
                  passphrase=None, vtn_fingerprint=None, show_fingerprint=True, ca_file=None,
-                 allow_jitter=True):
+                 allow_jitter=True, ven_id=None):
         """
         Initializes a new OpenADR Client (Virtual End Node)
 
@@ -60,13 +58,15 @@ class OpenADRClient:
                                      on startup. Defaults to True.
         :param str ca_file: The path to the PEM-formatted CA file for validating the VTN server's
                             certificate.
+        :param str ven_id: The ID for this VEN. If you leave this blank,
+                           a VEN_ID will be assigned by the VTN.
         """
 
         self.ven_name = ven_name
         if vtn_url.endswith("/"):
             vtn_url = vtn_url[:-1]
         self.vtn_url = vtn_url
-        self.ven_id = None
+        self.ven_id = ven_id
         self.registration_id = None
         self.poll_frequency = None
         self.vtn_fingerprint = vtn_fingerprint
@@ -117,7 +117,7 @@ class OpenADRClient:
         # if not hasattr(self, 'on_event'):
         #     raise NotImplementedError("You must implement on_event.")
         self.loop = asyncio.get_event_loop()
-        await self.create_party_registration()
+        await self.create_party_registration(ven_id=self.ven_id)
 
         if not self.ven_id:
             logger.error("No VEN ID received from the VTN, aborting.")
@@ -153,10 +153,6 @@ class OpenADRClient:
             self.scheduler.shutdown()
         if self.report_queue_task:
             self.report_queue_task.cancel()
-        if sys.version_info.minor > 8:
-            delayed_call_tasks = [task for task in asyncio.all_tasks() if task.get_name().startswith('DelayedCall')]
-            for task in delayed_call_tasks:
-                task.cancel()
         await self.client_session.close()
         await asyncio.sleep(0)
 
@@ -177,8 +173,7 @@ class OpenADRClient:
                    reading_type=enums.READING_TYPE.DIRECT_READ,
                    report_type=enums.REPORT_TYPE.READING, sampling_rate=None, data_source=None,
                    scale="none", unit=None, power_ac=True, power_hertz=50, power_voltage=230,
-                   market_context=None, end_device_asset_mrid=None, report_data_source=None,
-                   on_change=False):
+                   market_context=None, end_device_asset_mrid=None, report_data_source=None):
         """
         Add a new reporting capability to the client.
 
@@ -216,10 +211,6 @@ class OpenADRClient:
         :param str market_context: The Market Context that this report belongs to.
         :param str end_device_asset_mrid: the Meter ID for the end device that is measured by this report.
         :param report_data_source: A (list of) target(s) that this report is related to.
-        :param boolean on_change: Whether this measurement supports reporting when the value changes.
-                                  To use this functionality, call the ``client.report_on_change`` method
-                                  with parameters ``report_id, r_id, value``. You get the ``report_id``
-                                  and the ``r_id`` from the return of the ``client.add_report`` call.
         """
 
         # Verify input
@@ -242,11 +233,11 @@ class OpenADRClient:
         if sampling_rate is None:
             sampling_rate = objects.SamplingRate(min_period=timedelta(seconds=10),
                                                  max_period=timedelta(hours=24),
-                                                 on_change=on_change)
+                                                 on_change=False)
         elif isinstance(sampling_rate, timedelta):
             sampling_rate = objects.SamplingRate(min_period=sampling_rate,
                                                  max_period=sampling_rate,
-                                                 on_change=on_change)
+                                                 on_change=False)
 
         if data_collection_mode not in ('incremental', 'full'):
             raise ValueError("The data_collection_mode should be 'incremental' or 'full'.")
@@ -555,11 +546,6 @@ class OpenADRClient:
         Create a single report in response to a request from the VTN.
         """
 
-    def update_report_on_change(self, report_id, r_id, value):
-        """
-        Creates a new report after a value changed.
-        """
-
     async def update_report(self, report_request_id):
         """
         Call the previously registered report callback and send the result as a message to the VTN.
@@ -624,31 +610,13 @@ class OpenADRClient:
             expected_len = len(report_request['r_ids']) * int(report_interval / sampling_interval)
             if len(outgoing_report.intervals) == expected_len:
                 logger.info("The report is now complete with all the values. Will queue for sending.")
-                if self.allow_jitter:
-                    delay = random.uniform(0, min(30, report_interval / 2))
-                    if sys.version_info.minor >= 8:
-                        name = {'name': f'DelayedCall-OutgoingReport-{utils.generate_id()}'}
-                    else:
-                        name = {}
-                    self.loop.create_task(utils.delayed_call(func=self.pending_reports.put(outgoing_report),
-                                                             delay=delay), **name)
-                else:
-                    await self.pending_reports.put(self.incomplete_reports.pop(report_request_id))
+                await self.pending_reports.put(self.incomplete_reports.pop(report_request_id))
             else:
                 logger.debug("The report is not yet complete, will hold until it is.")
                 self.incomplete_reports[report_request_id] = outgoing_report
         else:
             logger.info("Report will be sent now.")
-            if self.allow_jitter:
-                delay = random.uniform(0, min(30, granularity.total_seconds() / 2))
-                if sys.version_info.minor >= 8:
-                    name = {'name': f'DelayedCall-OutgoingReport-{utils.generate_id()}'}
-                else:
-                    name = {}
-                self.loop.create_task(utils.delayed_call(func=self.pending_reports.put(outgoing_report),
-                                                         delay=delay), **name)
-            else:
-                await self.pending_reports.put(outgoing_report)
+            await self.pending_reports.put(outgoing_report)
 
     async def cancel_report(self, payload):
         """
@@ -659,19 +627,23 @@ class OpenADRClient:
         """
         A Queue worker that pushes out the pending reports.
         """
-
-        while True:
-            report = await self.pending_reports.get()
-            service = 'EiReport'
-            message = self._create_message('oadrUpdateReport', reports=[report])
-
-            try:
-                response_type, response_payload = await self._perform_request(service, message)
-            except Exception as err:
-                logger.error(f"Unable to send the report to the VTN. Error: {err}")
-            else:
-                if 'cancel_report' in response_payload:
-                    await self.cancel_report(response_payload['cancel_report'])
+        try:
+            while True:
+                report = await self.pending_reports.get()
+                service = 'EiReport'
+                message = self._create_message('oadrUpdateReport',
+                                               ven_id=self.ven_id,
+                                               request_id=utils.generate_id(),
+                                               reports=[report])
+                try:
+                    response_type, response_payload = await self._perform_request(service, message)
+                except Exception as err:
+                    logger.error(f"Unable to send the report to the VTN. Error: {err}")
+                else:
+                    if 'cancel_report' in response_payload:
+                        await self.cancel_report(response_payload['cancel_report'])
+        except asyncio.CancelledError:
+            return
 
     ###########################################################################
     #                                                                         #
@@ -704,6 +676,22 @@ class OpenADRClient:
         Placeholder for the on_register_report handler.
         """
 
+    ###########################################################################
+    #                                                                         #
+    #                             EMPTY RESPONSES                             #
+    #                                                                         #
+    ###########################################################################
+
+    async def send_response(self, service, response_code=200, response_description="OK", request_id=None):
+        """
+        Send an empty oadrResponse, for instance after receiving oadrRequestReregistration.
+        """
+        msg = self._create_message('oadrResponse',
+                                   response={'response_code': response_code,
+                                             'response_description': response_description,
+                                             'request_id': request_id})
+        await self._perform_request(service, msg)
+
     ###########################################################################
     #                                                                         #
     #                                  LOW LEVEL                              #
@@ -730,6 +718,8 @@ class OpenADRClient:
         except Exception as err:
             logger.error(f"Request error {err.__class__.__name__}:{err}")
             return None, {}
+        if len(content) == 0:
+            return None
         try:
             tree = validate_xml_schema(content)
             if self.vtn_fingerprint:
@@ -835,15 +825,18 @@ class OpenADRClient:
         if response_type is None:
             return
 
-        if response_type == 'oadrResponse':
-            logger.debug("No events or reports available")
+        elif response_type == 'oadrResponse':
+            logger.debug("Received empty response from the VTN.")
             return
 
-        if response_type == 'oadrRequestReregistration':
+        elif response_type == 'oadrRequestReregistration':
             logger.info("The VTN required us to re-register. Calling the registration procedure.")
+            await self.send_response(service='EiRegisterParty')
             await self.create_party_registration()
+            if self.reports:
+                await self.register_reports(self.reports)
 
-        if response_type == 'oadrDistributeEvent':
+        elif response_type == 'oadrDistributeEvent':
             if 'events' in response_payload and len(response_payload['events']) > 0:
                 await self._on_event(response_payload)
 
