@@ -22,10 +22,8 @@ from aiohttp import web
 from lxml.etree import XMLSyntaxError
 from signxml.exceptions import InvalidSignature
 
-from .. import errors
-from ..enums import STATUS_CODES
-from ..messaging import parse_message, validate_xml_schema, authenticate_message
-from ..utils import generate_id, get_cert_fingerprint_from_request
+from openleadr import enums, errors, hooks, utils
+from openleadr.messaging import parse_message, validate_xml_schema, authenticate_message
 
 from dataclasses import is_dataclass, asdict
 
@@ -49,9 +47,10 @@ class VTNService:
             content_type = request.headers.get('content-type', '')
             if not content_type.lower().startswith("application/xml"):
                 raise errors.HTTPError(response_code=HTTPStatus.BAD_REQUEST,
-                                       response_description="The Content-Type header must be application/xml, "
-                                                            "you provided {request.headers.get('content-type', '')}")
+                                       response_description="The Content-Type header must be application/xml; "
+                                                            f"you provided {request.headers.get('content-type', '')}")
             content = await request.read()
+            hooks.call('before_parse', content)
 
             # Validate the message to the XML Schema
             message_tree = validate_xml_schema(content)
@@ -59,24 +58,40 @@ class VTNService:
             # Parse the message to a type and payload dict
             message_type, message_payload = parse_message(content)
 
+            if message_type == 'oadrResponse':
+                raise errors.SendEmptyHTTPResponse()
+
             if 'vtn_id' in message_payload \
                     and message_payload['vtn_id'] is not None \
                     and message_payload['vtn_id'] != self.vtn_id:
                 raise errors.InvalidIdError(f"The supplied vtnID is invalid. It should be '{self.vtn_id}', "
                                             f"you supplied {message_payload['vtn_id']}.")
 
+            # Check if we know this VEN, ask for reregistration otherwise
+            if message_type not in ('oadrCreatePartyRegistration', 'oadrQueryRegistration') \
+                    and 'ven_id' in message_payload and hasattr(self, 'ven_lookup'):
+                result = await utils.await_if_required(self.ven_lookup(ven_id=message_payload['ven_id']))
+                if result is None or result.get('registration_id', None) is None:
+                    raise errors.RequestReregistration(message_payload['ven_id'])
+
             # Authenticate the message
             if request.secure and 'ven_id' in message_payload:
-                await authenticate_message(request, message_tree, message_payload,
-                                           self.fingerprint_lookup)
+                if hasattr(self, 'fingerprint_lookup'):
+                    await authenticate_message(request, message_tree, message_payload,
+                                               fingerprint_lookup=self.fingerprint_lookup)
+                elif hasattr(self, 'ven_lookup'):
+                    await authenticate_message(request, message_tree, message_payload,
+                                               ven_lookup=self.ven_lookup)
+                else:
+                    logger.error("Could not authenticate this VEN because "
+                                 "you did not provide a 'ven_lookup' function. Please see "
+                                 "https://openleadr.org/docs/server.html#signing-messages for info.")
 
             # Pass the message off to the handler and get the response type and payload
             try:
                 # Add the request fingerprint to the message so that the handler can check for it.
                 if request.secure and message_type == 'oadrCreatePartyRegistration':
-                    message_payload['fingerprint'] = get_cert_fingerprint_from_request(request)
-                if message_type == 'oadrCreatePartyRegistration':
-                    message_payload['ip_address'] = request.remote
+                    message_payload['fingerprint'] = utils.get_cert_fingerprint_from_request(request)
                 response_type, response_payload = await self.handle_message(message_type,
                                                                             message_payload)
             except Exception as err:
@@ -92,6 +107,17 @@ class VTNService:
             response_payload['vtn_id'] = self.vtn_id
             if 'ven_id' not in response_payload:
                 response_payload['ven_id'] = message_payload.get('ven_id')
+        except errors.RequestReregistration as err:
+            response_type = 'oadrRequestReregistration'
+            response_payload = {'ven_id': err.ven_id}
+            msg = self._create_message(response_type, **response_payload)
+            response = web.Response(text=msg,
+                                    status=HTTPStatus.OK,
+                                    content_type='application/xml')
+        except errors.SendEmptyHTTPResponse:
+            response = web.Response(text='',
+                                    status=HTTPStatus.OK,
+                                    content_type='application/xml')
         except errors.ProtocolError as err:
             # In case of an OpenADR error, return a valid OpenADR message
             response_type, response_payload = self.error_response(message_type,
@@ -127,9 +153,11 @@ class VTNService:
             response = web.Response(text=msg,
                                     status=HTTPStatus.OK,
                                     content_type='application/xml')
+        hooks.call('before_respond', response.text)
         return response
 
     async def handle_message(self, message_type, message_payload):
+        hooks.call('before_handle', message_type, message_payload)
         if message_type in self.handlers:
             handler = self.handlers[message_type]
             result = handler(message_payload)
@@ -151,15 +179,16 @@ class VTNService:
             response_payload['response'] = {'request_id': message_payload.get('request_id', None),
                                             'response_code': 200,
                                             'response_description': 'OK'}
-            response_payload['request_id'] = generate_id()
+            response_payload['request_id'] = utils.generate_id()
 
         else:
             response_type, response_payload = self.error_response('oadrResponse',
-                                                                  STATUS_CODES.COMPLIANCE_ERROR,
+                                                                  enums.STATUS_CODES.COMPLIANCE_ERROR,
                                                                   "A message of type "
                                                                   f"{message_type} should not be "
                                                                   "sent to this endpoint")
         logger.info(f"Responding to {message_type} with a {response_type} message: {response_payload}.")
+        hooks.call('after_handle', response_type, response_payload)
         return response_type, response_payload
 
     def error_response(self, message_type, error_code, error_description):
